class BaseController < ApplicationController
  # This class works as a Restfull CRUD Controller to provide the same
  # crud functions to all subcontrollers
  protect_from_forgery with: :exception
  respond_to :json, :html, :xml
  before_action :set_resource, only: [:show, :edit, :update, :destroy]
  before_action :disable_edition, only: [:show]
  before_action :load, except: [:index, :destroy]
  
  PER_PAGE = 6

  def index
    if included_attributes
      instance_variable_set(collection_instance_variable, model.includes(included_attributes).paginate(page: params[:page], per_page: PER_PAGE))
    else
      instance_variable_set(collection_instance_variable, model.paginate(page: params[:page], per_page: PER_PAGE))
    end
    search
    respond_with collection_variable
  end

  def show
  end

  def new
    instance_variable_set(instance_variable, model.new)
  end

  def edit
  end

  def create
    instance_variable_set(instance_variable, model.new(permit_params))
    flash[:success] = I18n.t('created', model: model.model_name.human) if variable.save
    responds
  end

  def update
    flash[:success] = I18n.t('updated', model: model.model_name.human) if variable.update(permit_params)
    responds
  end

  def destroy
    flash[:success] = I18n.t('destroyed', model: model.model_name.human) if variable.destroy
    redirect_to index_path
  end

  def template_not_found
    redirect_to '/404.html'
  end

  private
  def search
    return unless params[:search].present?
    result = collection_variable.search(params[:search])
    instance_variable_set(collection_instance_variable, result)
  end

  def variable
    instance_variable_get(instance_variable)
  end

  def collection_variable
    instance_variable_get(collection_instance_variable)
  end

  def controller_namespace
    params[:controller].split('/').first if params[:controller].include?('/')
  end

  def instance_variable
    "@#{model.name.underscore}"
  end

  def collection_instance_variable
    "@#{model.name.underscore.pluralize}"
  end

  def set_resource
    instance_variable_set(instance_variable, model.find(params[:id]))
  end

  def responds
    if controller_namespace
      respond_with controller_namespace.to_sym, variable
    else
      respond_with variable
    end
  end

  # to be used by subclasses
  def included_attributes
    nil
  end

  # to be used by subclasses
  def load
  end

  protected
  def model
    get_model_name.singularize.constantize
  end

  def index_path
    Rails.application.routes.url_helpers.send("#{get_model_name.underscore}_path")
  end

  def get_model_name
    model_name = self.class.name
    model_name.slice! 'Controller'
    model_name
  end

  def disable_edition
    @readonly = true
  end
end
