module LinksHelper

  def link_to_new(model, url, html_options = {})
    button_class = 'btn btn-default btn-lg'
    html_options.reverse_merge!(class: button_class)
    authorized = authorize_button :create, model, html_options, button_class if model.present?
    if authorized
      link_to t('new', model: model.model_name.human), url, html_options
    else
      content_tag(:div, class: 'tooltip-wrapper', "data-title" => I18n.t('not_authorized')) do
        link_to t('new', model: model.model_name.human), url, html_options
      end
    end
  end

  def link_to_edit(model, url, html_options = {})
    button_class = 'btn btn-default btn-sm'
    html_options.reverse_merge!(class: button_class)
    authorized = authorize_button :update, model, html_options, button_class if model.present?
    if authorized
      link_to t('edit'), url, html_options
    else
      content_tag(:div, class: 'tooltip-wrapper', "data-title" => I18n.t('not_authorized')) do
        link_to t('edit'), url, html_options
      end
    end
  end

  def link_to_show(model, url, html_options = {})
    button_class = 'btn btn-default btn-sm'
    html_options.reverse_merge!(class: button_class)
    authorized = authorize_button :read, model, html_options, button_class if model.present?
    if authorized
      link_to t('show'), url, html_options
    else
      content_tag(:div, class: 'tooltip-wrapper', "data-title" => I18n.t('not_authorized')) do
        link_to t('show'), url, html_options
      end
    end
  end

  def link_to_destroy(model, url, html_options = {})
    button_class = 'btn btn-danger glyphicon glyphicon-trash'
    html_options.reverse_merge!({class: button_class, "data-target" => "#delete-confirmation", "data-toggle" => "modal"})
    authorized = authorize_button :destroy, model, html_options, button_class if model.present?
    if authorized
      link_to '', url, html_options
    else
      content_tag(:div, class: 'tooltip-wrapper', "data-title" => I18n.t('not_authorized')) do
        link_to t('destroy'), url, html_options
      end
    end
  end

  def link_to_back(url, html_options = {})
    html_options.reverse_merge!(class: 'btn btn-default')
    link_to t('back'), url, html_options
  end

  def link_to_menu(model, url)
    link_to model.model_name.human.pluralize, url
  end

  def submit_or_edit_button(form, edit_path, is_edition)
    if is_edition && params[:action] != 'new'
      link_to_edit form.object, edit_path, class: "btn btn-primary btn-lg"
    else
      submit_button(form)
    end
  end

  def submit_button(form, value = nil)
    if value
      form.submit value, class: "btn btn-primary btn-lg"
    else
      form.submit t('save'), class: "btn btn-primary btn-lg"
    end
  end

  private
  def authorize_button(action, model, options, button_class)
    return true if Jabysoft.configuration.skip_authorize
    unless can? action, model
      options.merge!(class: button_class+" disabled ") and return false
    end
    true
  end
end
