module SearchableBehaviour
  extend ActiveSupport::Concern
  # It gives models search behaviour

  included do
    def self.search(params)
      if has_joins?
        joins(get_joins).where(query, p: "%#{params}%")
      else
        where(query, p: "%#{params}%")
      end
    end

    def self.look_for(*attrs)
      @attrs_to_search = attrs
    end

    def self.query
      return unless @attrs_to_search.any?
      each_attr = @attrs_to_search.map { |a| handle_attribute(a) + " LIKE :p "}
      each_attr.join("OR ")
    end

    def self.has_joins?
      select_hash_attributes.any?
    end

    def self.select_hash_attributes
      @attrs_to_search.select { |attribute| attribute.is_a? Hash }
    end

    def self.get_joins
      keys = []
      select_hash_attributes.each do |hash_attribute|
        keys |= hash_attribute.keys
      end
      keys
    end

    def self.handle_attribute(attribute)
      return "#{self.name.downcase.pluralize}.#{attribute}" unless attribute.is_a? Hash
      attribute.map {|k, v| "#{k.to_s.pluralize}.#{v}" }.first
    end

  end
end
